@@ -4,9 +4,9 @@
 #include "Imlib_private.h"
 
 /* uncomment this to compile imlib's cahce with pixmap accounting output */
-/* #define PIXMAP_ACCOUNTING */
+/*#define PIXMAP_ACCOUNTING*/
 /* uncomment this to compile imlib's cahce with image accounting output */
-/* #define IMAGE_ACCOUNTING */
+/*#define IMAGE_ACCOUNTING*/
 
 void
 dirty_pixmaps(ImlibData * id, ImlibImage * im)
@@ -171,10 +171,16 @@ free_image(ImlibData * id, ImlibImage * im)
 	  if (ptr->refnum)
 	    {
 	      ptr->refnum--;
-	      if (!ptr->refnum)
+	      if (ptr->refnum == 0)
 		{
 		  id->cache.num_image--;
 		  id->cache.used_image += ptr->im->rgb_width * ptr->im->rgb_height * 3;
+		  if (im->pixmap)
+		    {
+		      free_pixmappmap(id, im->pixmap);
+		      im->pixmap = 0;
+		      im->shape_mask = 0;
+		    }
 		}
 	    }
 	  return;
