@@ -143,10 +143,13 @@ Imlib_set_image_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * m
 {
   if ((!im) | (!mod))
     return;
-  im->mod.gamma = mod->gamma;
-  im->mod.brightness = mod->brightness;
-  im->mod.contrast = mod->contrast;
-  calc_map_tables(id, im);
+  if ((im->mod.gamma != mod->gamma) || (im->mod.brightness != mod->brightness) || (im->mod.contrast != mod->contrast))
+    {
+      im->mod.gamma = mod->gamma;
+      im->mod.brightness = mod->brightness;
+      im->mod.contrast = mod->contrast;
+      calc_map_tables(id, im);
+    }
 }
 
 void
@@ -154,10 +157,13 @@ Imlib_set_image_red_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier
 {
   if ((!im) | (!mod))
     return;
-  im->rmod.gamma = mod->gamma;
-  im->rmod.brightness = mod->brightness;
-  im->rmod.contrast = mod->contrast;
-  calc_map_tables(id, im);
+  if ((im->rmod.gamma != mod->gamma) || (im->rmod.brightness != mod->brightness) || (im->rmod.contrast != mod->contrast))
+    {
+      im->rmod.gamma = mod->gamma;
+      im->rmod.brightness = mod->brightness;
+      im->rmod.contrast = mod->contrast;
+      calc_map_tables(id, im);
+    }
 }
 
 void
@@ -165,10 +171,13 @@ Imlib_set_image_green_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifi
 {
   if ((!im) | (!mod))
     return;
-  im->gmod.gamma = mod->gamma;
-  im->gmod.brightness = mod->brightness;
-  im->gmod.contrast = mod->contrast;
-  calc_map_tables(id, im);
+  if ((im->gmod.gamma != mod->gamma) || (im->gmod.brightness != mod->brightness) || (im->gmod.contrast != mod->contrast))
+    {
+      im->gmod.gamma = mod->gamma;
+      im->gmod.brightness = mod->brightness;
+      im->gmod.contrast = mod->contrast;
+      calc_map_tables(id, im);
+    }
 }
 
 void
@@ -176,10 +185,13 @@ Imlib_set_image_blue_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifie
 {
   if ((!im) | (!mod))
     return;
-  im->bmod.gamma = mod->gamma;
-  im->bmod.brightness = mod->brightness;
-  im->bmod.contrast = mod->contrast;
-  calc_map_tables(id, im);
+  if ((im->bmod.gamma != mod->gamma) || (im->bmod.brightness != mod->brightness) || (im->bmod.contrast != mod->contrast))
+    {
+      im->bmod.gamma = mod->gamma;
+      im->bmod.brightness = mod->brightness;
+      im->bmod.contrast = mod->contrast;
+      calc_map_tables(id, im);
+    }
 }
 
 void
@@ -237,7 +249,7 @@ Imlib_set_image_red_curve(ImlibData * id, ImlibImage * im, unsigned char *mod)
       if (im->rmap[i] != mod[i])
 	{
 	  same = 0;
-	  i = 256;
+	  break;
 	}
     }
   if (same)
@@ -263,7 +275,7 @@ Imlib_set_image_green_curve(ImlibData * id, ImlibImage * im, unsigned char *mod)
       if (im->gmap[i] != mod[i])
 	{
 	  same = 0;
-	  i = 256;
+	  break;
 	}
     }
   if (same)
@@ -289,7 +301,7 @@ Imlib_set_image_blue_curve(ImlibData * id, ImlibImage * im, unsigned char *mod)
       if (im->bmap[i] != mod[i])
 	{
 	  same = 0;
-	  i = 256;
+	  break;
 	}
     }
   if (same)
@@ -372,6 +384,274 @@ Imlib_apply_modifiers_to_rgb(ImlibData * id, ImlibImage * im)
   dirty_pixmaps(id, im);
 }
 
+#define SHADE_PIXEL(pixel, dir, tmp) do {(tmp) = ((((double)pixel)/256.0) + ((dir) ? 0.2 : -0.2)) * 256.0; \
+                                         if ((tmp) > 255) (tmp) = 255; else if ((tmp) < 0) (tmp) = 0;} while (0)
+
+void
+Imlib_bevel_image(ImlibData *id, ImlibImage *im, ImlibBorder *bord, unsigned char up)
+{
+  register unsigned char *ptr;
+  register unsigned char r, g, b;
+  double v;
+  int x, y, xbound, ybound;
+
+  if ((!im) || (!bord))
+    return;
+
+  /* Left edge */
+  ptr = im->rgb_data + 3 * im->rgb_width * bord->top;
+  for (y = bord->top; y < im->rgb_height; y++)
+    {
+      xbound = im->rgb_height - y;
+      if (xbound > bord->left)
+	xbound = bord->left;
+      for (x = 0; x < xbound; x++)
+	{
+	  SHADE_PIXEL(*ptr, up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, up, v);
+	  *ptr++ = (unsigned char) v;
+	}
+      ptr += 3 * (im->rgb_width - xbound);
+    }
+
+  /* Right edge */
+  ybound = im->rgb_height - bord->bottom;
+  for (ptr = im->rgb_data, y = 0; y < ybound; y++)
+    {
+      xbound = bord->right - y;
+      if (xbound < 0)
+	xbound = 0;
+      ptr += 3 * (im->rgb_width - bord->right + xbound);
+      for (x = xbound; x < bord->right; x++)
+	{
+	  SHADE_PIXEL(*ptr, !up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, !up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, !up, v);
+	  *ptr++ = (unsigned char) v;
+	}
+    }
+
+  /* Top edge */
+  ptr = im->rgb_data;
+  for (y = 0; y < bord->top; y++)
+    {
+      xbound = im->rgb_width - y;
+      for (x = 0; x < xbound; x++)
+	{
+	  SHADE_PIXEL(*ptr, up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, up, v);
+	  *ptr++ = (unsigned char) v;
+	}
+      ptr += y * 3;
+    }
+
+  /* Bottom edge */
+  ptr = im->rgb_data + ((im->rgb_height - bord->bottom) * im->rgb_width * 3);
+  for (y = bord->bottom - 1; y >= 0; y--)
+    {
+      ptr += y * 3;
+      for (x = y; x < im->rgb_width; x++)
+	{
+	  SHADE_PIXEL(*ptr, !up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, !up, v);
+	  *ptr++ = (unsigned char) v;
+	  SHADE_PIXEL(*ptr, !up, v);
+	  *ptr++ = (unsigned char) v;
+	}
+    }
+}
+
+#undef SHADE_PIXEL
+#define SHADE_PIXEL(pixel, dir, tmp) do {(tmp) = ((((double)pixel)/depth_factor) + ((dir) ? 0.2 : -0.2)) * depth_factor; \
+                                         if ((tmp) > (depth_factor-1)) (tmp) = depth_factor - 1; else if ((tmp) < 0) (tmp) = 0;} while (0)
+#define MOD_PIXEL_LOW(x, y, up) do {v = XGetPixel(ximg, (x), (y)); r = (int) ctab[v & 0xff].r; g = (int) ctab[v & 0xff].g; b = (int) ctab[v & 0xff].b; \
+	                            SHADE_PIXEL(r, (up), dv); r = (int) dv; SHADE_PIXEL(g, (up), dv); g = (int) dv; SHADE_PIXEL(b, (up), dv); b = (int) dv; \
+	                            v = Imlib_best_color_match(id, &r, &g, &b); XPutPixel(ximg, (x), (y), v);} while (0)
+#define MOD_PIXEL_HIGH(x, y, up) do {v = XGetPixel(ximg, (x), (y)); r = (int) ((v >> br) & mr); g = (int) ((v >> bg) & mg); b = (int) ((v << bb) & mb); \
+	                             SHADE_PIXEL(r, (up), dv); r = (int) dv; SHADE_PIXEL(g, (up), dv); g = (int) dv; SHADE_PIXEL(b, (up), dv); b = (int) dv; \
+	                             v = ((r & mr) << br) | ((g & mg) << bg) | ((b & mb) >> bb); XPutPixel(ximg, (x), (y), v);} while (0)
+
+void
+Imlib_bevel_pixmap(ImlibData *id, Pixmap p, int w, int h, ImlibBorder *bord, unsigned char up)
+{
+
+  XImage *ximg;
+  register unsigned long i, v;
+  double dv;
+  short x, y, xbound, ybound;
+  unsigned int r, g, b;
+  ImlibColor ctab[256];
+  int real_depth = 0, depth_factor;
+  register int br, bg, bb;  /* Bitshifts */
+  register unsigned int mr, mg, mb;  /* Bitmasks */
+  XGCValues gcv;
+  GC gc;
+
+  if ((!id) || (!bord))
+    return;
+
+  depth_factor = 1 << id->x.depth;
+  if (id->x.depth <= 8) {
+
+    XColor cols[256];
+
+    for (i = 0; i < depth_factor; i++) {
+      cols[i].pixel = i;
+      cols[i].flags = DoRed | DoGreen | DoBlue;
+    }
+    XQueryColors(id->x.disp, id->x.root_cmap, cols, depth_factor);
+    for (i = 0; i < depth_factor; i++) {
+      ctab[i].r = cols[i].red >> 8;
+      ctab[i].g = cols[i].green >> 8;
+      ctab[i].b = cols[i].blue >> 8;
+      ctab[i].pixel = cols[i].pixel;
+    }
+  } else if (id->x.depth == 16) {
+
+    XWindowAttributes xattr;
+
+    XGetWindowAttributes(id->x.disp, id->x.root, &xattr);
+    if ((xattr.visual->red_mask == 0x7c00) && (xattr.visual->green_mask == 0x3e0) && (xattr.visual->blue_mask == 0x1f)) {
+      real_depth = 15;
+      depth_factor = 1 << 15;
+    }
+  }
+  if (!real_depth) {
+    real_depth = id->x.depth;
+  }
+  ximg = XGetImage(id->x.disp, p, 0, 0, w, h, -1, ZPixmap);
+  if (ximg == NULL) {
+    return;
+  }
+  if (id->x.depth <= 8) {
+    /* Left edge */
+    for (y = bord->top; y < h; y++)
+      {
+	xbound = h - y;
+	if (xbound > bord->left)
+	  xbound = bord->left;
+	for (x = 0; x < xbound; x++)
+	  {
+	    MOD_PIXEL_LOW(x, y, up);
+	  }
+      }
+
+    /* Right edge */
+    ybound = h - bord->bottom;
+    for (y = 0; y < ybound; y++)
+      {
+	xbound = bord->right - y;
+	if (xbound < 0)
+	  xbound = 0;
+	for (x = xbound; x < bord->right; x++)
+	  {
+	    MOD_PIXEL_LOW(x + (w - bord->right), y, !up);
+	  }
+      }
+
+    /* Top edge */
+    for (y = 0; y < bord->top; y++)
+      {
+	xbound = w - y;
+	for (x = 0; x < xbound; x++)
+	  {
+	    MOD_PIXEL_LOW(x, y, up);
+	  }
+      }
+
+    /* Bottom edge */
+    for (y = h - bord->bottom; y < h; y++)
+      {
+	for (x = h - y - 1; x < w; x++)
+	  {
+	    MOD_PIXEL_LOW(x, y, !up);
+	  }
+      }
+  } else {
+    /* Determine bitshift and bitmask values */
+    switch (real_depth) {
+      case 15:
+	br = 7;
+	bg = 2;
+	bb = 3;
+	mr = mg = mb = 0xf8;
+	break;
+      case 16:
+	br = 8;
+	bg = bb = 3;
+	mr = mb = 0xf8;
+	mg = 0xfc;
+	break;
+      case 24:
+      case 32:
+	br = 16;
+	bg = 8;
+	bb = 0;
+	mr = mg = mb = 0xff;
+	break;
+      default:
+	return;
+    }
+
+    /* Left edge */
+    for (y = bord->top; y < h; y++)
+      {
+	xbound = h - y;
+	if (xbound > bord->left)
+	  xbound = bord->left;
+	for (x = 0; x < xbound; x++)
+	  {
+	    MOD_PIXEL_HIGH(x, y, up);
+	  }
+      }
+
+    /* Right edge */
+    ybound = h - bord->bottom;
+    for (y = 0; y < ybound; y++)
+      {
+	xbound = bord->right - y;
+	if (xbound < 0)
+	  xbound = 0;
+	for (x = xbound; x < bord->right; x++)
+	  {
+	    MOD_PIXEL_HIGH(x + (w - bord->right), y, !up);
+	  }
+      }
+
+    /* Top edge */
+    for (y = 0; y < bord->top; y++)
+      {
+	xbound = w - y;
+	for (x = 0; x < xbound; x++)
+	  {
+	    MOD_PIXEL_HIGH(x, y, up);
+	  }
+      }
+
+    /* Bottom edge */
+    for (y = h - bord->bottom; y < h; y++)
+      {
+	for (x = h - y - 1; x < w; x++)
+	  {
+	    MOD_PIXEL_HIGH(x, y, !up);
+	  }
+      }
+  }
+  gc = XCreateGC(id->x.disp, p, 0, &gcv);
+  XPutImage(id->x.disp, p, gc, ximg, 0, 0, 0, 0, w, h);
+  XFreeGC(id->x.disp, gc);
+  XDestroyImage(ximg);
+}
+
 void
 Imlib_crop_image(ImlibData * id, ImlibImage * im, int x, int y, int w, int h)
 {
@@ -1115,6 +1395,8 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 
   if (!data)
     return NULL;
+  if (! id->x.disp)
+    return NULL;
   im = malloc(sizeof(ImlibImage));
   if (!im)
     return NULL;
@@ -2260,7 +2542,7 @@ Imlib_create_image_from_drawable(ImlibData * id, Drawable win, Pixmap mask,
 		    {
 		      pixel = XGetPixel(xim, xx, yy);
 		      r = (pixel >> 7) & 0xf8;
-		      g = (pixel >> 3) & 0xf8;
+		      g = (pixel >> 2) & 0xf8;
 		      b = (pixel << 3) & 0xf8;
 		      *ptr++ = r;
 		      *ptr++ = g;
