@@ -20,9 +20,9 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <math.h>
 #include <ctype.h>
 #include <time.h>
+#include <sys/types.h>
 #include <netinet/in.h>
 #ifdef HAVE_IPC_H
 #include <sys/ipc.h>
@@ -31,7 +31,6 @@
 #include <sys/shm.h>
 #endif
 #include <sys/time.h>
-#include <sys/types.h>
 
 #ifdef _HAVE_STRING_H
 #include <string.h>
@@ -49,6 +48,7 @@
 #include <X11/extensions/shape.h>
 #include <X11/cursorfont.h>
 #include <gdk/gdkprivate.h>
+#include <gdk/gdkx.h>
 
 #define BYTE_ORD_24_RGB 0
 #define BYTE_ORD_24_RBG 1
