@@ -4,4 +4,4 @@
 #include "gdk_imlib.h"
 #include "gdk_imlib_private.h"
 
-ImlibData          *_gdk_imlib_data;
+ImlibData          *_gdk_imlib_data = NULL;
