@@ -134,6 +134,7 @@ gdk_imlib_init()
   int                 loadpal;
   int                 vis;
   int                 newcm;
+  char               *old_locale;
 
   disp = (Display *) gdk_display;
   if (!disp)
@@ -233,7 +234,8 @@ gdk_imlib_init()
   id->num_colors = 0;
   homedir = getenv("HOME");
   g_snprintf(s, sizeof(s), "%s/.imrc", homedir);
-  setlocale(LC_ALL, "C");
+  old_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
+  setlocale(LC_NUMERIC, "C");
   f = fopen(s, "r");
   if (!f)
     f = fopen(SYSTEM_IMRC, "r");
@@ -402,7 +404,9 @@ gdk_imlib_init()
 	}
       fclose(f);
     }
-  setlocale(LC_ALL, "");
+  setlocale(LC_NUMERIC, old_locale);
+  g_free(old_locale);
+  
   /* list all visuals for the default screen */
   xvi.screen = id->x.screen;
   xvir = XGetVisualInfo(disp, VisualScreenMask, &xvi, &num);
@@ -629,6 +633,7 @@ gdk_imlib_init_params(GdkImlibInitParams * p)
   int                 loadpal;
   int                 vis;
   int                 newcm;
+  char               *old_locale;
 
   palfile = NULL;
   disp = (Display *) gdk_display;
@@ -723,7 +728,10 @@ gdk_imlib_init_params(GdkImlibInitParams * p)
   id->num_colors = 0;
   homedir = getenv("HOME");
   g_snprintf(s, sizeof(s), "%s/.imrc", homedir);
-  setlocale(LC_ALL, "C");
+  
+  old_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
+  setlocale(LC_NUMERIC, "C");
+  
   f = fopen(s, "r");
   if (!f)
     f = fopen(SYSTEM_IMRC, "r");
@@ -892,7 +900,9 @@ gdk_imlib_init_params(GdkImlibInitParams * p)
 	}
       fclose(f);
     }
-  setlocale(LC_ALL, "");
+  setlocale(LC_NUMERIC, old_locale);
+  g_free(old_locale);
+  
   if (p)
     {
       if (p->flags & PARAMS_VISUALID)
